@@ -6,7 +6,7 @@
 #' the transposition of a Filebacked Big Matrix.
 #'
 #' @inheritParams bigstatsr-package
-#' @inheritDotParams FBM -nrow -ncol -type -init
+#' @inheritParams FBM
 #'
 #' @return The new transposed Filebacked Big Matrix (or its descriptor). Its dimensions
 #' and type are automatically determined from the input Filebacked Big Matrix.
@@ -23,18 +23,17 @@
 #' Xt <- big_transpose(X)
 #' identical(t(X[]), Xt[])
 #'
-big_transpose <- function(X, ...) {
+big_transpose <- function(X, backingfile = tempfile()) {
 
   check_args(X = "assert_class(X, 'FBM')")
 
-  res <- FBM(ncol(X), nrow(X), typeof(X), init = NULL, ...)
+  res <- FBM(ncol(X), nrow(X), typeof(X), init = NULL,
+             backingfile = backingfile, create_bk = TRUE)
 
   transpose3(res, X)
 
-  save <- list(...)$save
   `if`(inherits(X, "FBM.code256"),
-       add_code256(res, code = X$code256,
-                   save = `if`(is.null(save), formals(FBM)$save, save)),
+       add_code256(res, code = X$code256),
        res)
 }
 
