@@ -35,11 +35,12 @@ big_univLinReg <- function(X, y.train,
                            thr.eigval = 1e-4,
                            ncores = 1) {
 
-  check_args()
+  check_args(y.train = "assert_multiple(y.train)")
 
   n <- length(ind.train)
   covar.train <- cbind(rep(1, n), covar.train)
   assert_lengths(ind.train, y.train, rows_along(covar.train))
+  stopifnot(n > ncol(covar.train))
 
   # get SVD of covar
   SVD <- svd(covar.train, nv = 0)
