@@ -3,9 +3,10 @@
 .onLoad <- function(libname, pkgname) {
   options(
     bigstatsr.ncores.max = parallel::detectCores(),
+    bigstatsr.cluster.type = "PSOCK",
     bigstatsr.check.args = TRUE,
     bigstatsr.block.sizeGB = 1,
-    bigstatsr.typecast.warning = TRUE
+    bigstatsr.downcast.warning = TRUE
   )
 }
 
@@ -14,9 +15,10 @@
 .onUnload <- function(libpath) {
   options(
     bigstatsr.ncores.max = NULL,
+    bigstatsr.cluster.type = NULL,
     bigstatsr.check.args = NULL,
     bigstatsr.block.sizeGB = NULL,
-    bigstatsr.typecast.warning = NULL
+    bigstatsr.downcast.warning = NULL
   )
 }
 
