@@ -9,15 +9,16 @@ using namespace Rcpp;
 /******************************************************************************/
 
 #define CALL_COPY_CDFIT_BINOMIAL_HSR(ACC, ACC_VAL) {                           \
-  return bigstatsr::biglassoLog::COPY_cdfit_binomial_hsr(ACC, y,               \
-    lambda, center, scale, resid, alpha, eps, max_iter, dfmax, warn,           \
-    ACC_VAL, y_val, n_abort, nlam_min);                                        \
+  return bigstatsr::biglassoLog::COPY_cdfit_binomial_hsr(ACC, y, base,         \
+    lambda, center, scale, resid, alpha, b0, eps, max_iter, dfmax, warn,       \
+    ACC_VAL, y_val, base_val, n_abort, nlam_min);                              \
 }
 
 // Dispatch function for COPY_cdfit_binomial_hsr
 // [[Rcpp::export]]
 List COPY_cdfit_binomial_hsr(Environment BM,
                              const NumericVector& y,
+                             const NumericVector& base,
                              const IntegerVector& row_idx,
                              const IntegerVector& col_idx,
                              const NumericMatrix& covar,
@@ -26,6 +27,7 @@ List COPY_cdfit_binomial_hsr(Environment BM,
                              const NumericVector& scale,
                              NumericVector& resid,
                              double alpha,
+                             double b0,
                              double eps,
                              int max_iter,
                              int dfmax,
@@ -33,6 +35,7 @@ List COPY_cdfit_binomial_hsr(Environment BM,
                              const IntegerVector& row_idx_val,
                              const NumericMatrix& covar_val,
                              const NumericVector& y_val,
+                             const NumericVector& base_val,
                              int n_abort,
                              int nlam_min) {
 
