@@ -5,17 +5,19 @@ Sys.unsetenv("R_TESTS")
 
 ################################################################################
 
-library(Matrix)
+library(testthat)
+library(bigstatsr)
 
 ################################################################################
 
-test_cores <- function() {
+opt.save <- options(bigstatsr.downcast.warning = FALSE,
+                    bigstatsr.block.sizeGB = 1e-5)
 
-  is.cran      <- !identical(Sys.getenv("BIGSTATSR_CRAN"), "false")
-  # is.randomSVD <- (get_reporter()$.context == "RANDOM_SVD")
+################################################################################
 
-  `if`(is.cran, 1, sample(2, size = 1))  # && is.randomSVD
-}
+not_cran <- identical(Sys.getenv("BIGSTATSR_CRAN"), "false")
+
+test_cores <- function() `if`(not_cran, sample(2, size = 1), 1)
 
 ################################################################################
 
@@ -43,6 +45,16 @@ diffPCs <- function(test, rot) {
 
 ################################################################################
 
-opt.save <- options(bigstatsr.typecast.warning = FALSE)
+set.seed(NULL)
+if (not_cran) {
+  # Seeds that won't work (because of bad luck)
+  # 3544 -> spLinReg & spLogReg
+  # 6649 -> spLinReg
+  do_not_use <- c(3544, 6649)
+  while ((SEED <- round(runif(1, 1, 9999))) %in% do_not_use) NULL
+} else {
+  SEED <- 1
+}
+cat("========== SEED:", SEED, "==========\n")
 
 ################################################################################
