@@ -2,6 +2,10 @@
 
 context("UNIV_LOG_REG")
 
+set.seed(SEED)
+
+################################################################################
+
 TOL <- 1e-4
 
 # Simulating some data
@@ -42,6 +46,9 @@ test_that("equality with glm with all data", {
 
       p <- plot(mod, type = sample(c("Manhattan", "Q-Q", "Volcano"), 1))
       expect_s3_class(p, "ggplot")
+
+      expect_error(predict(mod, abc = 2), "Argument 'abc' not used.")
+      expect_error(plot(mod, abc = 2), "Argument 'abc' not used.")
     }
   }
 })
